@@ -213,15 +213,15 @@ class ReportListActivity : AppCompatActivity(), ReportActivityListener {
                     val uri: Uri? = intent.data
                     if (uri != null) {
                         handleUri(uri)
+                        intent.putExtra(OPEN_INTENT_PROCESSED, true)
                     }
-                    intent.putExtra(OPEN_INTENT_PROCESSED, true)
                 }
                 Intent.ACTION_SEND -> {
                     val uri = intent.getParcelableExtra<Uri>(Intent.EXTRA_STREAM)
                     if (uri != null) {
                         handleUri(uri)
+                        intent.putExtra(OPEN_INTENT_PROCESSED, true)
                     }
-                    intent.putExtra(OPEN_INTENT_PROCESSED, true)
                 }
                 Intent.ACTION_SEND_MULTIPLE -> {
                     val uriList = intent.getParcelableArrayListExtra<Uri>(Intent.EXTRA_STREAM)
@@ -229,8 +229,8 @@ class ReportListActivity : AppCompatActivity(), ReportActivityListener {
                         for (i in uriList) {
                             handleUri(i)
                         }
+                        intent.putExtra(OPEN_INTENT_PROCESSED, true)
                     }
-                    intent.putExtra(OPEN_INTENT_PROCESSED, true)
                 }
             }
         }
@@ -545,8 +545,8 @@ class ReportListActivity : AppCompatActivity(), ReportActivityListener {
         reportModel = ViewModelProvider(this, viewModelFactory)[ReportViewModel::class.java]
 
         activityReportListBinding.addButton.setOnClickListener {
-            if (Build.VERSION.SDK_INT >= 19) {
-                val intent = Intent(Intent.ACTION_OPEN_DOCUMENT)
+            if (Build.VERSION.SDK_INT >= 20) { // Official Android FileChooser is buggy on Android 4.4 (19)
+                val intent = Intent(Intent.ACTION_GET_CONTENT)
 
                 intent.addCategory(Intent.CATEGORY_OPENABLE)
                 intent.type = "*/*"
