@@ -5,7 +5,6 @@
  */
 
 #include "mainwindow.h"
-#include "translate.h"
 #include "ui_mainwindow.h"
 
 #include "easyviewwidget.h"
@@ -16,7 +15,6 @@
 #include "sheet.h"
 #include "configtreetext.h"
 #include "custom.h"
-#include "graphplugin.h"
 
 #include <QDropEvent>
 #include <QFileDialog>
@@ -33,7 +31,10 @@
 #include <QToolBar>
 #include <QMainWindow>
 #include <QMenuBar>
-#include <QWebEngineView>
+#include <QTranslator>
+#include <QFontDatabase>
+#include <QPushButton>
+#include <QDesktopServices>
 /*
 #include <qwt/qwt_plot.h>
 #include <qwt/qwt_plot_curve.h>
@@ -48,7 +49,18 @@ using namespace ZenLib;
 #define QString2wstring(_DATA) \
     Ztring().From_UTF8(_DATA.toUtf8())
 
-#define VERSION "24.12"
+#define VERSION "25.04"
+
+#if defined(EDGE_WEBVIEW2_YES)
+#include "webview2widget.h"
+#include "graphplugin.h"
+#define WebViewWidget WebView2Widget
+#elif !defined(MEDIAINFO_HTML_NO)
+#include <QWebEngineView>
+#include "graphplugin.h"
+#define WebViewWidget QWebEngineView
+#endif
+
 #if defined(_WIN32) && defined(WINAPI_FAMILY) && (WINAPI_FAMILY==WINAPI_FAMILY_APP) //UWP Application
 #include <QtPlatformHeaders/QWindowsWindowFunctions>
 #include <ZenLib/Os_Utils.h>
@@ -170,7 +182,8 @@ MainWindow::MainWindow(QStringList filesnames, int viewasked, QWidget *parent) :
     setWindowTitle("MediaInfo");
 #endif
 
-    settings = new QSettings("MediaArea.net", "MediaInfo");
+    translator = new QTranslator(this);
+    settings = new QSettings("MediaArea.net", "MediaInfo", this);
     defaultSettings();
     applySettings();
 
@@ -199,11 +212,16 @@ MainWindow::MainWindow(QStringList filesnames, int viewasked, QWidget *parent) :
     ui->actionAbout->setIcon(QIcon(":/icon/about.svg"));
     ui->actionExport->setIcon(QIcon(":/icon/export.svg"));
 
-    menuView = new QMenu();
+    menuView = new QMenu(this);
     QActionGroup* menuItemGroup = new QActionGroup(this);
     for(int v=VIEW_EASY;v<NB_VIEW;v++) {
         QAction* action = new QAction(nameView((ViewMode)v), menuItemGroup);
         action->setCheckable(true);
+        #if defined(MEDIAINFO_HTML_NO)
+        if (v==VIEW_HTML || v==VIEW_GRAPH)
+            action->setEnabled(false);
+        #endif
+
         if(view==v)
             action->setChecked(true);
         action->setProperty("view",v);
@@ -249,6 +267,7 @@ MainWindow::MainWindow(QStringList filesnames, int viewasked, QWidget *parent) :
 
 MainWindow::~MainWindow()
 {
+    delete C;
     delete ui;
 }
 
@@ -293,7 +312,7 @@ void MainWindow::httpFinished()
     if (isNewer(wstring2QString(X("NewVersion")),QString(VERSION))) {
         qDebug() << "New version is available.";
         qDebug() << "latest is " << wstring2QString(X("NewVersion")).toStdString().c_str();
-        ui->menuBar->addAction(Tr("Update to new version"),this,SLOT(updateToNewVersion()));
+        ui->menuBar->addAction(tr("Update to new version"),this,SLOT(updateToNewVersion()));
     } else {
         qDebug() << "No new version available.";
         qDebug() << "latest is " << wstring2QString(X("NewVersion")).toStdString().c_str();
@@ -321,26 +340,26 @@ void MainWindow::toolBarOptions(QPoint p) {
     return;
 #else
     QMenu menu("toolbar options",ui->toolBar);
-    QMenu *menuI = new QMenu(Tr("Icon size"));
-    QMenu *menuT = new QMenu(Tr("Text position"));
+    QMenu *menuI = new QMenu(tr("Icon size"));
+    QMenu *menuT = new QMenu(tr("Text position"));
 
     menu.addMenu(menuT);
-    QString textsT[NBNAMES] = {Tr("Icons only"),Tr("Text only"),Tr("Text under icons"),Tr("Text beside icons")};
+    QString textsT[NBNAMES] = {tr("Icons only"),tr("Text only"),tr("Text under icons"),tr("Text beside icons")};
     QAction* actionsT[NBNAMES+1];
     Qt::ToolButtonStyle styles[NBNAMES] = {Qt::ToolButtonIconOnly,Qt::ToolButtonTextOnly,Qt::ToolButtonTextUnderIcon,Qt::ToolButtonTextBesideIcon};
     for(int i=0;i<NBNAMES;i++) {
         actionsT[i] = menuT->addAction(textsT[i]);
     }
-    actionsT[NBNAMES] = menuT->addAction(Tr("Default"));
+    actionsT[NBNAMES] = menuT->addAction(tr("Default"));
 
     menu.addMenu(menuI);
-    QString textsI[NBSIZES] = {Tr("Small"),Tr("Medium"),Tr("Big"),Tr("Huge")};
+    QString textsI[NBSIZES] = {tr("Small"),tr("Medium"),tr("Big"),tr("Huge")};
     QAction* actionsI[NBNAMES+1];
     QSize sizes[NBSIZES+1] = {QSize(16,16),QSize(22,22),QSize(32,32),QSize(48,48)};
     for(int i=0;i<NBSIZES;i++) {
         actionsI[i] = menuI->addAction(textsI[i]+"("+QString::number(sizes[i].width())+"x"+QString::number(sizes[i].height())+")");
     }
-    actionsI[NBSIZES] = menuI->addAction(Tr("Default"));
+    actionsI[NBSIZES] = menuI->addAction(tr("Default"));
 
     QAction* a = menu.exec(ui->toolBar->mapToGlobal(p));
     if(a) {
@@ -409,9 +428,28 @@ QDir MainWindow::getCommonDir(Core*C) {
 void MainWindow::changeEvent(QEvent *e)
 {
     QMainWindow::changeEvent(e);
+    QFile file;
     switch (e->type()) {
     case QEvent::LanguageChange:
         ui->retranslateUi(this);
+        // force refresh of text of view button on toolbar
+        buttonView->setText(tr("View"));
+        ui->toolBar->setToolButtonStyle(Qt::ToolButtonStyle(Qt::ToolButtonIconOnly));
+        ui->toolBar->setToolButtonStyle(Qt::ToolButtonStyle(settings->value("iconStyle",Qt::ToolButtonIconOnly).toInt()));
+        // update texts of view menu items
+        for(int v=VIEW_EASY;v<NB_VIEW;v++) {
+            ui->menuView->actions().at(v)->setText(nameView((ViewMode)v));
+        }
+        // set MediaInfoLib language
+        file.setFileName(":/languages/Plugin/Language/" + settings->value("language", "en").toString() + ".csv");
+        if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            QTextStream in(&file);
+            QString fileContent = in.readAll();
+            C->MI->Option_Static(__T("Language"), fileContent.toStdWString());
+            file.close();
+        }
+        // refresh titlebar and MediaInfoLib output
+        refreshDisplay();
         break;
     default:
         break;
@@ -429,7 +467,21 @@ void MainWindow::openFiles(QStringList fileNames) {
     //Configuring
     if(fileNames.isEmpty())
         return;
-    for(int i=0;i<fileNames.size();i++) {
+    // Process input paths/filenames/URLs
+    for (int i = 0; i < fileNames.size(); ++i) {
+        // Skip processing if is a network URL
+        QUrl url(fileNames[i]);
+        if (url.isValid())
+            if (!url.scheme().isEmpty())
+                if (url.scheme().startsWith("http", Qt::CaseInsensitive) ||
+                    url.scheme().startsWith("ftp", Qt::CaseInsensitive) ||
+                    url.scheme().startsWith("sftp", Qt::CaseInsensitive))
+                    continue;
+        // Resolve to target if is a Windows .lnk shortcut
+        QFileInfo fileInfo(fileNames[i]);
+        if (fileInfo.suffix().compare("lnk", Qt::CaseInsensitive) == 0 && !fileInfo.symLinkTarget().isEmpty())
+            fileNames[i] = fileInfo.symLinkTarget();
+        // Convert directory separators to native separators for file paths
         fileNames[i] = QDir::toNativeSeparators(fileNames[i]);
     }
     C->Menu_File_Open_Files_Begin(settings->value("closeBeforeOpen",true).toBool(), true);
@@ -442,14 +494,14 @@ void MainWindow::openFiles(QStringList fileNames) {
 
 void MainWindow::openTimerInit ()
 {
-    progressDialog=new QProgressDialog(Tr("Opening files..."), Tr("Abort Opening"), 0, 10000, this);
+    progressDialog=new QProgressDialog(tr("Opening files..."), tr("Abort Opening"), 0, 10000, this);
     progressDialog->setWindowModality(Qt::WindowModal);
     progressDialog->setMinimumDuration(0);
     progressDialog->setWindowTitle("MediaInfo");
 
     if (timer==NULL)
     {
-        timer=new QTimer();
+        timer=new QTimer(this);
         connect(timer, SIGNAL(timeout()), this, SLOT(updateProgressBar()));
         timer->start(100);
     }
@@ -490,8 +542,13 @@ void MainWindow::openDir(QString dirName) {
 }
 
 void MainWindow::refreshDisplay() {
-    QStringList fonts = { "Cascadia Mono", "Mono" };
-    QFont font(fonts);
+    QFont font;
+    if (!settings->value("monoFont", "").toString().isEmpty()) {
+        font.fromString(settings->value("monoFont", "").toString());
+        font.setFamilies(QStringList({ font.family(), "Cascadia Mono", "Courier New", "Malgun Gothic", "MS Gothic", "NSimSun", "Mono" }));
+    } else {
+        font.setFamilies(QStringList({ "Cascadia Mono", "Courier New", "Malgun Gothic", "MS Gothic", "NSimSun", "Mono" }));
+    }
     font.setStyleHint(QFont::TypeWriter);
 
     ui->actionAdapt_columns_to_content->setVisible(false);
@@ -499,7 +556,7 @@ void MainWindow::refreshDisplay() {
     QWidget* viewWidget;
     ui->actionExport->setEnabled(C->Count_Get()>0);
     ui->actionClose_All->setEnabled(C->Count_Get()>0);
-    QDomDocument* xis;
+    QDomDocument* xis{nullptr};
 
     C->Menu_Option_Preferences_Option(__T("Enable_Ffmpeg"), settings->value("enableFFmpeg",false).toBool() ? __T("1") : __T("0"));
 
@@ -517,10 +574,10 @@ void MainWindow::refreshDisplay() {
 
     if(C->Count_Get()<=0) {
 #if defined(_WIN32) && defined(WINAPI_FAMILY) && WINAPI_FAMILY==WINAPI_FAMILY_APP //UWP Application
-        viewWidget = new QLabel(Tr("You must at least open 1 file.\nOpen a file or a directory."));
+        viewWidget = new QLabel(tr("You must at least open 1 file.\nOpen a file or a directory."));
 #else
-        viewWidget = new QLabel(Tr("You must at least open 1 file.\nOpen a file or a directory, or simply drag n drop files in the window."));
-        setWindowTitle(Tr("MediaInfo"));
+        viewWidget = new QLabel(tr("You must at least open 1 file.\nOpen a file or a directory, or simply drag n drop files in the window."));
+        setWindowTitle(tr("MediaInfo"));
 #endif
         ((QLabel*)viewWidget)->setAlignment(Qt::AlignCenter);
     }
@@ -676,13 +733,16 @@ void MainWindow::refreshDisplay() {
                 viewWidget = new EasyViewWidget(C);
                 break;
             case VIEW_HTML:
-                C->Menu_View_HTML();
-                viewWidget = new QWebEngineView();
-                ((QWebEngineView*)viewWidget)->setHtml(wstring2QString(C->Inform_Get()));
+                #ifndef MEDIAINFO_HTML_NO
+                    C->Menu_View_HTML();
+                    viewWidget = new WebViewWidget();
+                    ((WebViewWidget*)viewWidget)->setHtml(wstring2QString(C->Inform_Get()));
+                #endif
                 break;
             case VIEW_GRAPH:
-                viewWidget = new QWebEngineView();
-                ((QWebEngineView*)viewWidget)->setHtml(Generate_Graph_HTML(C, settings));
+                #ifndef MEDIAINFO_HTML_NO
+                    viewWidget = new GraphViewWidget(C, settings);
+                #endif
                 break;
             case VIEW_TREE:
                 C->Menu_View_Tree();
@@ -690,7 +750,7 @@ void MainWindow::refreshDisplay() {
                 break;
             case VIEW_SHEET:
                 C->Menu_View_Sheet();
-                viewWidget = new SheetView(C,this);
+                viewWidget = new SheetView(C,this,&font);
                 ui->actionReset_field_sizes->setVisible(true);
                 if(!Sheet::getSheet()->getAdaptColumns())
                     ui->actionAdapt_columns_to_content->setVisible(true);
@@ -707,18 +767,48 @@ void MainWindow::refreshDisplay() {
         if(C->Count_Get()==1)
             setWindowTitle("MediaInfo - "+shortName(C,wstring2QString(C->Get(0, Stream_General, 0, __T("CompleteName")))));
         else
-            setWindowTitle(QString("MediaInfo - %1 ").arg((int)C->Count_Get())+Tr("files","window title"));
+            setWindowTitle(QString("MediaInfo - %1 ").arg((int)C->Count_Get())+tr("files","window title"));
 #endif
     }
-    setCentralWidget(viewWidget);
+
+    // Check if conformance glossary button should show
+    bool showGlossaryButton = false;
+    if (C->Count_Get() > 0) {
+        size_t FilePos = 0;
+        size_t AudioCount = C->Count_Get(FilePos, Stream_Audio, -1);
+        for (size_t AudioPos = 0; AudioPos < AudioCount; ++AudioPos) {
+            if (C->Get(FilePos, Stream_Audio, AudioPos, __T("ConformanceErrors")) != __T("") ||
+                C->Get(FilePos, Stream_Audio, AudioPos, __T("ConformanceWarnings")) != __T("")) {
+                showGlossaryButton = true;
+                break;
+            }
+        }
+    }
+    if (showGlossaryButton) {
+        // Show conformance glossary button below viewWidget
+        QWidget* compositeViewWidget = new QWidget(this);
+        QVBoxLayout* vLayout = new QVBoxLayout(compositeViewWidget);
+        QPushButton* footerButton = new QPushButton(tr("Go to conformance errors and warnings glossary page"));
+        connect(footerButton, SIGNAL(clicked()), this, SLOT(footerButtonClicked()));
+        vLayout->addWidget(viewWidget);
+        vLayout->addWidget(footerButton);
+        compositeViewWidget->setLayout(vLayout);
+        setCentralWidget(compositeViewWidget);
+    }
+    else
+        // Show just the viewWidget
+        setCentralWidget(viewWidget);
+
+    //Delete object(s)
+    if (xis) delete xis;
 }
 
 QTreeWidget* MainWindow::showTreeView(bool completeDisplay) {
     QTreeWidget* treeWidget = new QTreeWidget();
     //treeWidget->setHeaderHidden(true);
     treeWidget->setColumnCount(2);
-    QStringList headers = QStringList(Tr("key"));
-    headers.append(Tr("value"));
+    QStringList headers = QStringList(tr("key"));
+    headers.append(tr("value"));
     treeWidget->setHeaderLabels(headers);
     unsigned fileCount = (unsigned)C->Count_Get();
     QDir dir = getCommonDir(C);
@@ -883,14 +973,14 @@ void MainWindow::defaultSettings() {
     if(Sheet::getNbSheets()==0) {
         Sheet::add("example");
         Sheet::setDefault(0);
-        Sheet::getSheet()->addColumn(Tr("File Name").toStdString().c_str(),300,Stream_General,"CompleteName");
-        Sheet::getSheet()->addColumn(Tr("Format").toStdString().c_str(),100,Stream_General,"Format");
-        Sheet::getSheet()->addColumn(Tr("Video Codec List").toStdString().c_str(),100,Stream_General,"Video_Codec_List");
-        Sheet::getSheet()->addColumn(Tr("Audio Codec List").toStdString().c_str(),100,Stream_General,"Audio_Codec_List");
-        Sheet::getSheet()->addColumn(Tr("Text Codec List").toStdString().c_str(),100,Stream_General,"Text_Codec_List");
-        Sheet::getSheet()->addColumn(Tr("Video Format").toStdString().c_str(),100,Stream_Video,"Format");
-        Sheet::getSheet()->addColumn(Tr("Audio Duration").toStdString().c_str(),100,Stream_Audio,"Duration");
-        Sheet::getSheet()->addColumn(Tr("Text Width").toStdString().c_str(),100,Stream_Text,"Width");
+        Sheet::getSheet()->addColumn(tr("File Name").toStdString().c_str(),300,Stream_General,"CompleteName");
+        Sheet::getSheet()->addColumn(tr("Format").toStdString().c_str(),100,Stream_General,"Format");
+        Sheet::getSheet()->addColumn(tr("Video Codec List").toStdString().c_str(),100,Stream_General,"Video_Codec_List");
+        Sheet::getSheet()->addColumn(tr("Audio Codec List").toStdString().c_str(),100,Stream_General,"Audio_Codec_List");
+        Sheet::getSheet()->addColumn(tr("Text Codec List").toStdString().c_str(),100,Stream_General,"Text_Codec_List");
+        Sheet::getSheet()->addColumn(tr("Video Format").toStdString().c_str(),100,Stream_Video,"Format");
+        Sheet::getSheet()->addColumn(tr("Audio Duration").toStdString().c_str(),100,Stream_Audio,"Duration");
+        Sheet::getSheet()->addColumn(tr("Text Width").toStdString().c_str(),100,Stream_Text,"Width");
     }
     ConfigTreeText::load(settings);
     if(ConfigTreeText::getNbConfigTreeTexts()==0) {
@@ -915,6 +1005,10 @@ void MainWindow::applySettings() {
     ui->toolBar->setToolButtonStyle(Qt::ToolButtonStyle(settings->value("iconStyle",Qt::ToolButtonIconOnly).toInt()));
     ui->toolBar->setIconSize(settings->value("iconSize",QSize(32,32)).toSize());
     C->Menu_Option_Preferences_Option(__T("LegacyStreamDisplay"), settings->value("legacyStreamDisplay",false).toBool() ? __T("1") : __T("0"));
+
+    qApp->removeTranslator(translator);
+    Q_UNUSED(translator->load(settings->value("language", "en").toString(), ":/languages/Translations"));
+    qApp->installTranslator(translator);
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
@@ -984,7 +1078,7 @@ void MainWindow::on_actionOpen_triggered()
         fileNames += Path;
     }
 #else
-    fileNames = QFileDialog::getOpenFileNames(this, Tr("Open File(s)"), getCommonDir(C).absolutePath());
+    fileNames = QFileDialog::getOpenFileNames(this, tr("Open File(s)"), getCommonDir(C).absolutePath());
 #endif
     openFiles(fileNames);
 
@@ -1031,7 +1125,7 @@ void MainWindow::on_actionOpen_Folder_triggered()
 
     dirName = Path;
 #else
-    dirName = QFileDialog::getExistingDirectory(this,Tr("Open Folder"), getCommonDir(C).absolutePath());
+    dirName = QFileDialog::getExistingDirectory(this,tr("Open Folder"), getCommonDir(C).absolutePath());
 #endif
     openDir(dirName);
 
@@ -1085,6 +1179,7 @@ void MainWindow::on_actionPreferences_triggered()
         applySettings();
         if(settings->value("defaultView",VIEW_EASY)!=oldView) {
             this->view = (ViewMode)settings->value("defaultView",VIEW_EASY).toInt();
+            refreshDisplay();
         }
     }
 }
@@ -1224,6 +1319,12 @@ void MainWindow::on_actionFull_Parse_toggled(bool checked)
     refreshDisplay();
 }
 
+void MainWindow::on_actionTrace_toggled(bool checked)
+{
+    C->MI->Option_Static(__T("Trace_Level"), checked?__T("1"):__T("0"));
+    ui->menuView->actions().at(VIEW_TEXT)->trigger();
+}
+
 void MainWindow::on_actionClose_All_triggered()
 {
     C->Menu_File_Open_Files_Begin(true);
@@ -1245,3 +1346,12 @@ void MainWindow::buttonViewClicked()
     menuView->exec(buttonView->mapToGlobal(QPoint(0, 0)));
 #endif
 }
+
+void MainWindow::footerButtonClicked()
+{
+    const Ztring Inform_Save = C->Menu_Option_Preferences_Option(__T("Inform_Get"), __T(""));
+    C->Menu_Option_Preferences_Inform(__T("Conformance_JSON"));
+    const QString URL = wstring2QString(C->Inform_Get());
+    C->Menu_Option_Preferences_Inform(Inform_Save);
+    QDesktopServices::openUrl(QUrl(URL));
+}
