@@ -21,6 +21,7 @@
 #include <QMenu>
 #include <QToolButton>
 #include <QUrl>
+#include <QTranslator>
 #ifdef NEW_VERSION
     #include <QNetworkAccessManager>
     #include <QNetworkReply>
@@ -67,6 +68,7 @@ private:
     QTextBrowser* showCustomView(bool forcePlainText=false);
     QToolButton* buttonView;
     QMenu* menuView;
+    QTranslator* translator;
 
     Ui::MainWindow *ui;
     //Non-GUI Elements
@@ -88,6 +90,7 @@ private slots:
     void on_actionClose_All_triggered();
     void on_actionAdvanced_Mode_toggled(bool);
     void on_actionFull_Parse_toggled(bool);
+    void on_actionTrace_toggled(bool);
     void on_actionExport_triggered();
     void on_actionPreferences_triggered();
     void on_actionKnown_parameters_triggered();
@@ -100,6 +103,7 @@ private slots:
     void on_actionOpen_triggered();
     void toolBarOptions(QPoint);
     void buttonViewClicked();
+    void footerButtonClicked();
 #ifdef NEW_VERSION
     void httpFinished();
     void httpReadyRead();
