@@ -285,8 +285,6 @@ __fastcall TMainF::TMainF(TComponent* Owner)
             I->Open(Ztring().From_Local(ParamStr(I1).c_str()));
     #endif
 
-    //Refresh after opening files
-    Refresh();
 }
 
 //***************************************************************************
@@ -294,6 +292,16 @@ __fastcall TMainF::TMainF(TComponent* Owner)
 //***************************************************************************
 
 //---------------------------------------------------------------------------
+void __fastcall TMainF::FormShow(TObject *Sender)
+{
+    if (!StartupReady) {
+        //Refresh once on startup after opening files
+        StartupReady = true;
+        Refresh();
+    }
+}
+//---------------------------------------------------------------------------
+
 void __fastcall TMainF::GUI_Configure()
 {
     //Load Configuration
@@ -807,6 +815,9 @@ void __fastcall TMainF::Translate()
 //---------------------------------------------------------------------------
 void __fastcall TMainF::Refresh(TTabSheet *Page)
 {
+    if (!StartupReady)
+        return;
+
     if (Page==NULL)
         Page=this->Page->ActivePage;
     size_t FilesCount=I->Count_Get();
@@ -1106,7 +1117,7 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
             InstallFolder = InstallFolder.substr(0, InstallFolder.rfind(__T("\\")) + 1);
 
             Ztring State=I->Option_Static(__T("Info_Graph_Svg_Plugin_State"), __T(""));
-            if (State == __T("0") || !File::Exists(InstallFolder+__T("\\Plugin\\Graph\\Template.html"))) //Try to install plugin
+            if (State == __T("0") || !File::Exists(InstallFolder+__T("\\Plugin\\Graph\\version.txt"))) //Try to install plugin
             {
                 TPluginF* P = new TPluginF(this, PLUGIN_GRAPH);
                 if (P->Configure())
